var MicroTaskQueue,
  bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

MicroTaskQueue = (function() {
  function MicroTaskQueue() {
    this.flush = bind(this.flush, this);
    this.cancel = bind(this.cancel, this);
    this.queue = bind(this.queue, this);
    this.tasks = [];
    this.max_recursion = 5000;
    this.__next_index = 0;
    this.__next_handle = 1;
    this.schedule = (function(callback) {
      var div;
      div = document.createElement('div');
      new MutationObserver(callback).observe(div, {
        attributes: true
      });
      return function() {
        div.classList.toggle('foo');
      };
    })(this.flush);
  }

  MicroTaskQueue.prototype.queue = function(task) {
    if (!this.tasks.length) {
      this.schedule();
    }
    this.tasks.push(task);
    return this.__next_handle++;
  };

  MicroTaskQueue.prototype.cancel = function(handle) {
    var index;
    index = handle - (this.__next_handle - this.tasks.length);
    if (index > this.__next_index) {
      return this.tasks[index] = null;
    }
  };

  MicroTaskQueue.prototype.flush = function() {
    var count, err, mark, task;
    count = 0;
    mark = 0;
    while (this.__next_index < this.tasks.length) {
      if (!(task = this.tasks[this.__next_index])) {
        this.__next_index++;
        continue;
      }
      if (this.__next_index > mark) {
        if (count++ > this.max_recursion) {
          console.error('Exceeded max task recursion');
          this.__next_index = 0;
          return this.tasks = [];
        }
        mark = this.tasks.length;
      }
      try {
        task();
      } catch (error) {
        err = error;
        console.error(err);
      }
      this.__next_index++;
    }
    this.__next_index = 0;
    this.tasks = [];
  };

  return MicroTaskQueue;

})();

module.exports = new MicroTaskQueue();
